@@ -5,7 +5,7 @@ HyPhy - Hypothesis Testing Using Phylogenies.
 Copyright (C) 1997-now
 Core Developers:
   Sergei L Kosakovsky Pond (spond@ucsd.edu)
-  Art FY Poon    (apoon@cfenet.ubc.ca)
+  Art FY Poon    (apoon42@uwo.ca)
   Steven Weaver (sweaver@ucsd.edu)
   
 Module Developers:
@@ -44,7 +44,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "list.h"
 #include "avllistx.h"
 #include "hy_strings.h"
-#include "errorfns.h"
+#include "hy_string_buffer.h"
 #include "stdio.h"
 #include "classes.h"
 
@@ -59,6 +59,9 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "variable.h"
 #include "variablecontainer.h"
 #include "trie.h"
+#include "hbl_env.h"
+#include "global_things.h"
+
 
 //End parser specific includes
 
@@ -92,34 +95,32 @@ extern      _String         HalfOps;
 
 extern      _Trie           UnOps;
 
-extern      long            printDigits,
-                            verbosityLevel;
 
-extern      long            lastMatrixDeclared,
-            subNumericValues;
+extern      long            lastMatrixDeclared;
 
 long        LocateVarByName (_String const&);
-_Variable*  LocateVar       (long index);
-_Variable*  FetchVar        (long index);
-_PMathObj   FetchObjectFromVariableByType       (_String const*, const unsigned long, long = -1, _String* = nil);
-_PMathObj   FetchObjectFromVariableByTypeIndex  (long, const unsigned long, long = -1, _String* = nil);
-_PMathObj   FetchObjectFromFormulaByType         (_Formula&, const unsigned long, long = -1, _String* = nil);
+inline _Variable*  LocateVar       (long index) {
+        return (_Variable *)(((BaseRef*)variablePtrs.list_data)[index]);
+}
+HBLObjectRef   FetchObjectFromVariableByType       (_String const*, const unsigned long, long = -1, _String* = nil);
+HBLObjectRef   FetchObjectFromVariableByTypeIndex  (long, const unsigned long, long = -1, _String* = nil);
+HBLObjectRef   FetchObjectFromFormulaByType         (_Formula&, const unsigned long, long = -1, _String* = nil);
 _String     FetchObjectNameFromType (const unsigned long);
 _String const&    AppendContainerName     (_String const&, _VariableContainer const*);
 _String const&    AppendContainerName     (_String const&, _String const*);
 _String*    FetchMathObjectNameOfTypeByIndex (const unsigned long objectClass, const long objectIndex);
 
 void        DeleteVariable  (_String const&, bool deleteself = true);
-void        DeleteVariable  (long, bool deleteself);
+void        DeleteVariable  (long, bool deleteself, bool do_checks = true);
 
 void        DeleteTreeVariable
 (_String&, _SimpleList&,bool);
 
 
 
-void        stashParameter  (_String const& name, _Parameter  newVal, bool);
-void        setParameter    (_String const& name, _Parameter def, _String* = nil);
-void        setParameter    (_String const& name, _PMathObj  def, _String* = nil, bool = true);
+void        stashParameter  (_String const& name, hyFloat  newVal, bool);
+void        setParameter    (_String const& name, hyFloat def, _String* = nil);
+void        setParameter    (_String const& name, HBLObjectRef  def, _String* = nil, bool = true);
 
 long        VerbosityLevel (void);
 void        ReplaceVar      (_Variable*);
@@ -131,36 +132,39 @@ bool        ExpressionCalculator(void);
 bool        ExpressionCalculator(_String data);
 
 _Variable*  CheckReceptacle
-(_String const*, _String const & , bool = true, bool = false);
+(_String const*, _String const & , bool = true, bool = false, bool clear_trees = true);
 
 bool        CheckReceptacleAndStore
-(_String const*,_String, bool, _PMathObj, bool = true);
+(_String const*,_String, bool, HBLObjectRef, bool = true);
 
 bool        CheckReceptacleAndStore
-(_String,_String, bool, _PMathObj, bool = true);
+(_String,_String, bool, HBLObjectRef, bool = true);
+
+_Variable*  CheckReceptacleCommandIDException
+(_String const* name, const long id, bool checkValid, bool isGlobal = false, _ExecutionList* context = nil);
 
 _Variable*  CheckReceptacleCommandID
 (_String const* name, const long id, bool checkValid, bool isGlobal = false, _ExecutionList* context = nil);
 
 bool        CheckReceptacleCommandIDAndStore
-(_String const* name, const long id, bool checkValid, _PMathObj v, bool dup = true, bool isGlobal = false);
+(_String const* name, const long id, bool checkValid, HBLObjectRef v, bool dup = true, bool isGlobal = false);
 
 void        FinishDeferredSF(void);
 
 void        SetupOperationLists (void);
 void        ExportIndVariables
-(_String&, _String&, _SimpleList*);
+(_StringBuffer&, _StringBuffer&, _SimpleList*);
 void        ExportDepVariables
-(_String&, _String&, _SimpleList*);
+(_StringBuffer&, _StringBuffer&, _SimpleList*);
 void        ExportCatVariables
-(_String&, _SimpleList*);
+(_StringBuffer&, _SimpleList*);
 
 void        SplitVariablesIntoClasses
 (_SimpleList&, _SimpleList&, _SimpleList&, _SimpleList&);
 
-
-extern      _Parameter machineEps;
-bool        CheckEqual      (_Parameter,_Parameter, _Parameter = machineEps);
+bool        CheckEqual      (hyFloat,hyFloat,hyFloat = hy_global::kMachineEpsilon);
+bool        CheckRange      (hyFloat value,hyFloat lb, hyFloat ub, bool exeptions = false);
+bool        CheckArgumentType  (HBLObjectRef object, long type, bool exceptions = false);
 
 extern      _AVLListX       _hyApplicationGlobals;
 
@@ -174,43 +178,44 @@ extern      _AVLListX       _hyApplicationGlobals;
 */
 void        SplitVariableIDsIntoLocalAndGlobal (const _SimpleList& inList, _List& outList);
 
-_Parameter  AddNumbers  (_Parameter, _Parameter);
-_Parameter  SubNumbers  (_Parameter, _Parameter);
-_Parameter  MultNumbers (_Parameter, _Parameter);
-_Parameter  AndNumbers  (_Parameter, _Parameter);
-_Parameter  DivNumbers  (_Parameter, _Parameter);
-_Parameter  EqualNumbers(_Parameter, _Parameter);
-_Parameter  LessThan    (_Parameter, _Parameter);
-_Parameter  GreaterThan (_Parameter, _Parameter);
-_Parameter  LessThanE   (_Parameter, _Parameter);
-_Parameter  GreaterThanE(_Parameter, _Parameter);
-_Parameter  Power       (_Parameter, _Parameter);
-_Parameter  RandomNumber(_Parameter, _Parameter);
-_Parameter  ExpNumbers  (_Parameter);
-_Parameter  LogNumbers  (_Parameter);
-_Parameter  AbsNumber   (_Parameter);
-_Parameter  MinusNumber (_Parameter);
-_Parameter  MaxNumbers  (_Parameter, _Parameter);
-_Parameter  MinNumbers  (_Parameter, _Parameter);
-_Parameter  FastMxAccess(Ptr, _Parameter);
-void        FastMxWrite (Ptr, _Parameter, _Parameter);
+hyFloat  AddNumbers  (hyFloat, hyFloat);
+hyFloat  SubNumbers  (hyFloat, hyFloat);
+hyFloat  MultNumbers (hyFloat, hyFloat);
+hyFloat  AndNumbers  (hyFloat, hyFloat);
+hyFloat  DivNumbers  (hyFloat, hyFloat);
+hyFloat  EqualNumbers(hyFloat, hyFloat);
+hyFloat  LessThan    (hyFloat, hyFloat);
+hyFloat  GreaterThan (hyFloat, hyFloat);
+hyFloat  LessThanE   (hyFloat, hyFloat);
+hyFloat  GreaterThanE(hyFloat, hyFloat);
+hyFloat  Power       (hyFloat, hyFloat);
+hyFloat  RandomNumber(hyFloat, hyFloat);
+hyFloat  ExpNumbers  (hyFloat);
+hyFloat  LogNumbers  (hyFloat);
+hyFloat  AbsNumber   (hyFloat);
+hyFloat  MinusNumber (hyFloat);
+hyFloat  MaxNumbers  (hyFloat, hyFloat);
+hyFloat  MinNumbers  (hyFloat, hyFloat);
+hyFloat  FastMxAccess(hyPointer, hyFloat);
+void        FastMxWrite (hyPointer, hyFloat, hyFloat);
 
-BaseRef parameterToString       (_Parameter);
-void    parameterToCharBuffer   (_Parameter, char*, long, bool json = false);
+BaseRef parameterToString       (hyFloat);
+void    parameterToCharBuffer   (hyFloat, char*, long, bool json = false);
 
-_Parameter  InterpolateValue        (_Parameter*, _Parameter*, long, _Parameter*, _Parameter*, _Parameter, _Parameter&);
-_Parameter  TrapezoidLevelK         (_Formula&, _Variable*, _Parameter, _Parameter, long);
-_Parameter  TrapezoidLevelKSimple   (_Formula&, _Variable*, _Parameter, _Parameter, long, _SimpleFormulaDatum*, _SimpleFormulaDatum*, _SimpleList&, _SimpleList&);
+hyFloat  InterpolateValue        (hyFloat*, hyFloat*, long, hyFloat*, hyFloat*, hyFloat, hyFloat&);
+hyFloat  TrapezoidLevelK         (_Formula&, _Variable*, hyFloat, hyFloat, long);
+hyFloat  TrapezoidLevelKSimple   (_Formula&, _Variable*, hyFloat, hyFloat, long, _SimpleFormulaDatum*, _SimpleFormulaDatum*, _SimpleList&, _SimpleList&);
 
 
 void        PopulateArraysForASimpleFormula
 (_SimpleList&, _SimpleFormulaDatum*);
 
-void        WarnNotDefined (_PMathObj, long, _hyExecutionContext* );
-void        WarnWrongNumberOfArguments (_PMathObj, long, _hyExecutionContext*, _List *);
+void        WarnNotDefined (HBLObjectRef, long, _hyExecutionContext* );
+void        WarnWrongNumberOfArguments (HBLObjectRef, long, _hyExecutionContext*, _List *);
   
-extern      _Parameter  pi_const;
+extern      hyFloat  pi_const, tolerance;
+
+
 extern      bool        useGlobalUpdateFlag;
-extern      _String     noneToken;
 
 #endif
