@@ -5,7 +5,7 @@
  Copyright (C) 1997-now
  Core Developers:
  Sergei L Kosakovsky Pond (sergeilkp@icloud.com)
- Art FY Poon    (apoon@cfenet.ubc.ca)
+ Art FY Poon    (apoon42@uwo.ca)
  Steven Weaver (sweaver@temple.edu)
  
  Module Developers:
@@ -37,10 +37,6 @@
  
  */
 
-#include "list.h"
-#include "hy_strings.h"
-#include "errorfns.h"
-#include "parser.h"
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -49,9 +45,12 @@
 #include <limits.h>
 #include <stdarg.h>
 
-#ifdef    __HYPHYDMALLOC__
-#include "dmalloc.h"
-#endif
+#include "list.h"
+#include "hy_strings.h"
+#include "hy_string_buffer.h"
+#include "parser.h"
+
+using namespace hy_global;
 
 
 /*
@@ -61,18 +60,15 @@ Constructors
 */
 
 // Does nothing
-_List::_List ()
-{
+_List::_List () {
 }
 
 // Length constructor
-_List::_List (unsigned long l):_SimpleList(l)
-{
+_List::_List (unsigned long l):_SimpleList(l) {
 }
 
 // Stack Copy contructor
-_List::_List (const _List& l, long from, long to)
-{
+_List::_List (const _List& l, long from, long to) {
     if (from == 0 && to == -1) { // copy the whole thing
         BaseRef   br = (BaseRef)&l;
         Duplicate (br);
@@ -81,16 +77,15 @@ _List::_List (const _List& l, long from, long to)
         NormalizeCoordinates (from, to, l.lLength);
 
         for (long i = from; i <= to; i++) {
-            (*this) << ((BaseRef*)l.lData)[i];
+            (*this) << ((BaseRef*)l.list_data)[i];
         }
     }
 }
 
 // Construct a list of substrings from the original string separated by char
-_List::_List (BaseRef ss, char sep)
-{
+_List::_List (BaseRef ss, char sep) {
     _String* s = (_String*)ss;
-    if (s->Length()!=0) {
+    if (s->empty() == false) {
         long cp=0,cpp;
         while ((cpp = s->Find(sep,cp,-1))!=-1) {
             if (cpp>cp) {
@@ -106,26 +101,23 @@ _List::_List (BaseRef ss, char sep)
 }
 
 // Data constructor (1 member list)
-_List::_List (BaseRef br) {
-    lLength = 1;
-    laLength = MEMORYSTEP;
-    lData = (long*)MemAllocate (laLength * sizeof(Ptr));
-    BaseRef   object_copy = br->makeDynamic();
-    ((BaseRef*)lData)[0]= object_copy;
+_List::_List (BaseRef br) : _SimpleList (0L) {
+    ((BaseRef*)list_data)[0]= br->makeDynamic();
 }
 
 
 //Destructor
-_List::~_List(void)
-{
-    if (nInstances<=1) {
-        for (unsigned long i = 0; i<lLength; i++) {
-            BaseRef t = ((BaseRef*)lData)[i];
-            if (t) {
-                if (t->nInstances<=1) {
-                    DeleteObject(t);
+_List::~_List(void) {
+    if (CanFreeMe()) {
+        BaseRef * references =(BaseRef*)list_data;
+    
+        for (unsigned long i = 0UL; i<lLength; i++) {
+            BaseRef an_object = references[i];
+            if (an_object) {
+                if (an_object->CanFreeMe()) {
+                    DeleteObject(an_object);
                 } else {
-                    t->nInstances--;
+                    an_object->RemoveAReference();
                 }
             }
         }
@@ -141,32 +133,27 @@ Operator Overloads
 */
 
 // Element location functions (0,llength - 1)
-BaseRef& _List::operator [] (long i)
-{
-    BaseRef t = BaseRef(_SimpleList::operator[] (i));
-    if (t)
-        if (t->nInstances>1) {
-            t->nInstances--;
-            ((BaseRef*)(lData))[i]=t->makeDynamic();
+BaseRef& _List::operator [] (long i) {
+    // TODO 20170426: SLKP, maybe deprecate
+    BaseRef ith_object = GetItem(i);
+    if (ith_object)
+        if (!ith_object->CanFreeMe()) {
+            ith_object->RemoveAReference();
+            ((BaseRef*)(list_data))[i]=ith_object->makeDynamic();
         }
 
-    return ((BaseRef*)(lData))[i];
+    return ((BaseRef*)(list_data))[i];
 }
 
 // Element location functions (0,llength - 1)
-BaseRef _List::operator () (const unsigned long i)
-{
-    return ((BaseRef*)lData)[i];
+BaseRef _List::operator () (const unsigned long i) {
+    return ((BaseRef*)list_data)[i];
 }
 
-// Element location functions (0,llength - 1)
-BaseRef _List::GetItem (const unsigned long i) const {
-    return ((BaseRef*)lData)[i];
-}
 
   // Element location functions (0,llength - 1)
 BaseRef _List::GetItemRangeCheck(const unsigned long i) const {
-  return i < lLength ? ((BaseRef*)lData)[i] : nil;
+  return i < lLength ? ((BaseRef*)list_data)[i] : nil;
 }
 
 
@@ -174,12 +161,10 @@ BaseRef _List::GetItemRangeCheck(const unsigned long i) const {
 const _List _List::operator = (const _List& l) {
 
     Clear(true);
-  
     lLength = l.lLength;
     RequestSpace(laLength);
     for (unsigned long i = 0UL; i<lLength; i++) {
-        ((BaseRef*)(lData))[i] = l.GetItem (i);
-        ((BaseRef*)(lData))[i] -> AddAReference();
+        (((BaseRef*)(list_data))[i] = l.GetItem (i))-> AddAReference();
     }
 
     return *this;
@@ -197,14 +182,10 @@ const _List _List::operator & (_List const & l) const
 {
     _List res (l.lLength + lLength);
   
-    if (!res.laLength) {
-        return res;
-    }
-
-    BaseRef * res_data = (BaseRef*) res.lData;
+    BaseRef * res_data = (BaseRef*) res.list_data;
   
-    if (lData) {
-      BaseRef * base_data = (BaseRef*) lData;
+    if (list_data) {
+      BaseRef * base_data = (BaseRef*) list_data;
       
       for (res.lLength = 0UL; res.lLength < lLength; res.lLength++) {
         res_data [res.lLength] = base_data[res.lLength];
@@ -212,8 +193,8 @@ const _List _List::operator & (_List const & l) const
       }
     }
   
-    if (l.lData) {
-      BaseRef * l_data = (BaseRef*) l.lData;
+    if (l.list_data) {
+      BaseRef * l_data = (BaseRef*) l.list_data;
       for (unsigned long li = 0UL; li < l.lLength; li++, res.lLength++) {
         res_data [res.lLength] = l_data [li];
         l_data [li] -> AddAReference();
@@ -227,13 +208,10 @@ const _List _List::operator & (_List const & l) const
 const _List _List::operator & (BaseRef br) const
 {
     _List res (lLength+1UL);
-    if (res.laLength == 0UL) {
-        return res;
-    }
   
-    BaseRef * res_data = (BaseRef*) res.lData;
-    if (lData) {
-      BaseRef * base_data = (BaseRef*) lData;
+    BaseRef * res_data = (BaseRef*) res.list_data;
+    if (list_data) {
+      BaseRef * base_data = (BaseRef*) list_data;
       
       for (res.lLength = 0UL; res.lLength < lLength; res.lLength++) {
         res_data [res.lLength] = base_data[res.lLength];
@@ -260,18 +238,18 @@ _List& _List::operator < (BaseRef br) {
   //  InsertElement (br, -1, false);
   lLength++;
   if (lLength>laLength) {
-    unsigned long incBy = (MEMORYSTEP*5 > lLength)? MEMORYSTEP: lLength/5;
-    
+    unsigned long incBy = (MEMORYSTEP*4 > lLength)? MEMORYSTEP: (lLength << 2);
     laLength+=incBy;
-    
-    if (lData) {
-      checkPointer (lData = (long*)MemReallocate((char*)lData, laLength*sizeof(void*)));
+    _EnsureCorrectStorageType();
+    /*
+    if (list_data) {
+      list_data = (long*)MemReallocate((char*)list_data, laLength*sizeof(void*));
     } else {
-      checkPointer (lData = (long*)MemAllocate(laLength*sizeof(void*)));
-    }
+      list_data = (long*)MemAllocate(laLength*sizeof(void*));
+    }*/
   }
   
-  ((BaseRef*)lData)[lLength-1]=br;
+  ((BaseRef*)list_data)[lLength-1]=br;
   return *this;
 }
 
@@ -282,14 +260,14 @@ _List& _List::operator << (BaseRef br) {
 
 _List& _List::operator << (_List const& source) {
   for (unsigned long k=0UL; k<source.lLength; k++) {
-      (*this) << ((BaseRef*)source.lData)[k];
+      (*this) << ((BaseRef*)source.list_data)[k];
   }
   return *this;
 }
 
 _List& _List::operator < (_List const& source) {
   for (unsigned long k=0UL; k<source.lLength; k++) {
-    (*this) << ((BaseRef*)source.lData)[k];
+    (*this) < ((BaseRef*)source.list_data)[k];
   }
   return *this;
 }
@@ -305,13 +283,13 @@ Methods
 ==============================================================
 */
 
-void _List::AppendNewInstance (BaseRef br) {
+long _List::AppendNewInstance (BaseRef br) {
     if (br) {
-        (*this)<<br;
-        br->RemoveAReference();
+        (*this)<br;
     } else {
-        checkPointer (br);
+        HandleApplicationError(_String ("Passed a null reference to ") & __PRETTY_FUNCTION__);
     }
+    return lLength;
 }
 
 
@@ -329,14 +307,13 @@ long  _List::BinaryFindObject (BaseObj const * s, long startAt) const {
   
     while (top>bottom) {
         middle = (top+bottom)/2;
-        _String* stp = (_String*)(((BaseRef*)lData)[middle]->toStr());
-
-        int      cres = st->Compare (stp);
+        _String* stp = (_String*)(((BaseRef*)list_data)[middle]->toStr());
+        hyComparisonType      cres = st->Compare (*stp);
         DeleteObject (stp);
 
-        if (cres < 0) {
+        if (cres == kCompareLess) {
             top = middle==top?top-1:middle;
-        } else if (cres == 0) {
+        } else if (cres == kCompareEqual) {
             return middle;
         } else {
             bottom = middle==bottom?bottom+1:middle;
@@ -344,8 +321,8 @@ long  _List::BinaryFindObject (BaseObj const * s, long startAt) const {
 
     }
     middle = top;
-    _String* stp=(_String*)(((BaseRef*)lData)[middle]->toStr());
-    if (st->Equal(stp)) {
+    _String* stp=(_String*)(((BaseRef*)list_data)[middle]->toStr());
+    if (st->Equal(*stp)) {
         DeleteObject(stp);
         return middle;
     }
@@ -353,8 +330,7 @@ long  _List::BinaryFindObject (BaseObj const * s, long startAt) const {
     return -middle-2;
 }
 
-long  _List::BinaryInsert (BaseRef s)
-{
+long  _List::BinaryInsert (BaseRef s) {
 
     if (!lLength) {
         InsertElement (s,0,true);
@@ -365,49 +341,51 @@ long  _List::BinaryInsert (BaseRef s)
     if (pos<0) {
         return -pos+2;
     }
-    _String *s1 = (_String*)s->toStr(), *s2 =(_String*) ((*this)(pos))->toStr();
-    if (*s2<*s1) {
-        pos++;
+    
+    if (pos < lLength) {
+        _String *s1 = (_String*)s->toStr(),
+                *s2 = (_String*) ((*this)(pos))->toStr();
+        if (*s2<*s1) {
+            pos++;
+        }
+        DeleteObject(s1);
+        DeleteObject(s2);
     }
-    DeleteObject(s1);
-    DeleteObject(s2);
     InsertElement (s,pos,true);
     return pos>=lLength?lLength-1:pos;
 
 }
 
-void    _List::bumpNInst (void)
-{
-    for (unsigned long i = 0; i<lLength; i++) {
-        ((BaseRef*)lData)[i]->nInstances++;
+void    _List::bumpNInst (void) {
+    for (unsigned long i = 0UL; i<lLength; i++) {
+        ((BaseRef*)list_data)[i]->AddAReference();
     }
 }
 
-void  _List::Clear (bool completeClear)
-{
-    if (nInstances<=1) {
+void  _List::Clear (bool completeClear) {
+    if (CanFreeMe()) {
         for (unsigned long i = 0UL; i<lLength; i++) {
-            DeleteObject (((BaseRef*)lData)[i]);
+            DeleteObject (((BaseRef*)list_data)[i]);
         }
         _SimpleList::Clear(completeClear);
 
     } else {
-        nInstances--;
+        RemoveAReference();
     }
 }
 
-long  _List::Compare (long i, long j) const {
-    _String             *si = (_String*)lData[i],
-                         *sj = (_String*)lData[j];
+hyComparisonType  _List::Compare (long i, long j) const {
+    _String             *si = (_String*)list_data[i],
+                         *sj = (_String*)list_data[j];
 
-    return  si->Compare(sj);
+    return  si->Compare(*sj);
 }
 
-long  _List::Compare (BaseObj const * i, long j) const {
-    _String const       *sj = (_String const*)lData[j],
+hyComparisonType  _List::Compare (BaseObj const * i, long j) const {
+    _String const       *sj = (_String const*)list_data[j],
                         *si = (_String const*)i;
 
-    return  si->Compare(sj);
+    return  si->Compare(*sj);
 }
 
 unsigned long _List::Count()
@@ -420,23 +398,17 @@ void  _List::Delete (long index, bool delete_object)
 {
     if (index>=0 && index<lLength) {
         if (delete_object) {
-            DeleteObject (((BaseRef*)lData)[index]);
+            DeleteObject (((BaseRef*)list_data)[index]);
         }
         lLength--;
         if (lLength-index)
             for (unsigned long i = index; i < lLength; i++) {
-                lData[i] = lData[i+1];
+                list_data[i] = list_data[i+1];
             }
-        //memcpy ((Ptr)lData+sizeof(BaseRef)*(index),(Ptr)lData+sizeof(BaseRef)*(index+1),sizeof(BaseRef)*(lLength-index));
-    }
-    if (laLength-lLength>MEMORYSTEP) {
-        laLength -= ((laLength-lLength)/MEMORYSTEP)*MEMORYSTEP;
-        if (laLength > 0)
-          lData = (long*)MemReallocate ((char*)lData, laLength*sizeof(Ptr));
-        else {
-          free (lData); lData = nil;
-        }
+        //memcpy ((hyPointer)list_data+sizeof(BaseRef)*(index),(hyPointer)list_data+sizeof(BaseRef)*(index+1),sizeof(BaseRef)*(lLength-index));
     }
+    
+    _UpdateStorageType();
 
 }
 
@@ -446,35 +418,28 @@ void  _List::DeleteList (const _SimpleList& toDelete)
     if (toDelete.lLength) {
         long k = 0;
         for (unsigned long i = 0; i<lLength; i++) {
-            if (k<toDelete.lLength && i==toDelete.lData[k]) {
-                DeleteObject (((BaseRef*)lData)[i]);
+            if (k<toDelete.lLength && i==toDelete.list_data[k]) {
+                DeleteObject (((BaseRef*)list_data)[i]);
                 //if (k<toDelete.lLength)
                 k++;
             } else {
-                ((BaseRef*)lData)[i-k] = ((BaseRef*)lData)[i];
+                ((BaseRef*)list_data)[i-k] = ((BaseRef*)list_data)[i];
             }
         }
-        lLength -= toDelete.lLength;
-        if (laLength-lLength>MEMORYSTEP) {
-            laLength -= ((laLength-lLength)/MEMORYSTEP)*MEMORYSTEP;
-            if (laLength > 0)
-              lData = (long*)MemReallocate ((char*)lData, laLength*sizeof(Ptr));
-            else {
-              free (lData); lData = nil;
-            }
-          }
+        lLength -= k;
+        _UpdateStorageType();
+
     }
 }
 
-void    _List::Duplicate (const BaseRef theRef)
-{
+void    _List::Duplicate (BaseRefConst theRef) {
+    _List::Clear();
     _SimpleList::Duplicate (theRef);
-    if (lData)
-        for (unsigned long i = 0; i<lLength; i++) {
-            if (((BaseRef*)lData)[i]) {
-                (((BaseRef*)lData)[i])->nInstances++;
-            }
+    for (unsigned long i = 0UL; i<lLength; i++) {
+        if (((BaseRef*)list_data)[i]) {
+            (((BaseRef*)list_data)[i])->AddAReference();
         }
+    }
 }
 
 bool _List::Equal(_List const & l2) const
@@ -483,10 +448,11 @@ bool _List::Equal(_List const & l2) const
         return false;
     }
 
-    for (unsigned long i=0; i<lLength; i++)
-        if (!((_String*)lData[i])->Equal ((_String*)l2.lData[i])) {
+    for (unsigned long i=0; i<lLength; i++) {
+        if (!((_String*)list_data[i])->Equal (*(_String*)l2.list_data[i])) {
             return false;
         }
+    }
 
     return true;
 }
@@ -494,82 +460,44 @@ bool _List::Equal(_List const & l2) const
 long  _List::FindObject (BaseRefConst s, long startat) const {
     _String const * st = (_String const*)s;
     for (unsigned long i = startat; i<lLength; i++) {
-        _String * sp = (_String*)(((BaseRef*)lData)[i]->toStr());
+      
+        _String * sp = (_String*)(((BaseRef*)list_data)[i]->toStr());
 
-        if (st->Equal(sp)) {
+        if (*st == *sp) {
             DeleteObject(sp);
             return i;
         }
         DeleteObject(sp);
     }
-    return -1;
+    return kNotFound;
 }
 
-long  _List::FindString (BaseRef s, long startat, bool caseSensitive, long upTo)
-{
-    char * s1, *s2;
-    long t = ((_String*)s)->sLength;
-
-    if (upTo < 0 || upTo >= lLength) {
-        upTo = lLength-1;
-    }
-
-    for (long i = startat; i<= upTo; i++) {
-        s1 = ((_String*)s)->sData;
-        if (((_String*)(((BaseRef*)lData)[i]))->sLength==t) {
-            s2 = ((_String*)(((BaseRef*)lData)[i]))->sData;
-            long j = 0;
-            if (caseSensitive)
-                for (j=0; (*s1==*s2)&&(j<t); j++,s1++,s2++) ;
-            else
-                for (j=0; (toupper(*s1)==toupper(*s2))&&(j<t); j++,s1++,s2++) ;
-
-            if (j==t) {
-                return i;
-            }
-        }
-    }
-    return -1;
-}
-
-long  _List::FreeUpMemory (long requestedBytes)
-{
-    long freed = 0;
-    for (unsigned long i = 0; i<lLength; i++) {
-        BaseRef t = ((BaseRef*)lData)[i];
-        freed+=t->FreeUpMemory(requestedBytes-freed);
-        if (freed>=requestedBytes) {
-            return freed;
-        }
-    }
-    return freed;
-}
 
 // Append & store operator
 void _List::InsertElement (BaseRef br, long insertAt, bool store, bool pointer) {
     _SimpleList::InsertElement (br, insertAt, store, pointer);
 }
 
-void    _List::Intersect (_List& l1, _List& l2, _SimpleList* idx, _SimpleList* idx2)
-{
+void    _List::Intersect (_List& l1, _List& l2, _SimpleList* idx, _SimpleList* idx2) {
     // compute the union of two sorted lists
     // each repeat appears exactly once
-    if (lLength) {
+    if (nonempty()) {
         Clear();
     }
 
-    long  c1 = 0,
-          c2 = 0;
+    long  c1 = 0L,
+          c2 = 0L;
 
     while (c1<l1.lLength && c2<l2.lLength) {
-        while (c1<l1.lLength && ((_String*)l1(c1))->Compare((_String*)l2(c2))<0) {
+        while (c1<l1.lLength && ((_String*)l1(c1))->Compare(*(_String*)l2(c2)) == kCompareLess) {
             c1++;
         }
+      
         if (c1==l1.lLength) {
             break;
         }
 
-        while (c1<l1.lLength && c2<l2.lLength &&((_String*)l1(c1))->Equal((_String*)l2(c2))) {
+        while (c1<l1.lLength && c2<l2.lLength &&((_String*)l1(c1))->Equal(*(_String*)l2(c2))) {
             if (idx) {
                 (*idx) << c1;
             }
@@ -583,14 +511,14 @@ void    _List::Intersect (_List& l1, _List& l2, _SimpleList* idx, _SimpleList* i
         if (c1==l1.lLength || c2==l2.lLength) {
             break;
         }
-        while (c2<l2.lLength && ((_String*)l2(c2))->Compare((_String*)l1(c1))<0) {
+        while (c2<l2.lLength && ((_String*)l2(c2))->Compare(*(_String*)l1(c1)) == kCompareLess) {
             c2++;
         }
     }
 }
 
-BaseRef  _List::Join (BaseRefConst spacer, long startAt, long endAt) {
-    _String *joined = new _String (256L,true);
+_String*  _List::Join (_String const & spacer, long startAt, long endAt) const {
+    _StringBuffer *joined = new _StringBuffer (256L);
     if (endAt < 0) { 
         endAt = lLength; 
     } else if (endAt > lLength) {
@@ -598,56 +526,44 @@ BaseRef  _List::Join (BaseRefConst spacer, long startAt, long endAt) {
     }
 
     for (unsigned long k = MAX(0L,startAt); k < endAt; k++) {
-        if (k) {
-            (*joined) << *(_String const*)spacer;
+        if (k > startAt) {
+            (*joined) << spacer;
         }
-        joined->AppendNewInstance((_String*) ((BaseRef*)lData)[k]->toStr());
+        joined->AppendNewInstance((_String*) ((BaseRef*)list_data)[k]->toStr());
     }
 
-    joined->Finalize();
     return joined;
 }
 
-void _List::Place (BaseRef br)
-{
+void _List::Place (BaseRef br) {
 //  InsertElement (br, -1, false);
     lLength++;
     if (lLength>laLength) {
         laLength+=MEMORYSTEP;
-        if (lData) {
-            checkPointer (lData = (long*)MemReallocate((char*)lData, laLength*sizeof(void*)));
-        } else {
-            checkPointer (lData = (long*)MemAllocate(laLength*sizeof(void*)));
-        }
+        _EnsureCorrectStorageType();
     }
-    ((BaseRef*)lData)[lLength-1]=br;
+    ((BaseRef*)list_data)[lLength-1]=br;
 }
 
 //TODO: makeDynamic should be MakeDynamic to follow convention.
-BaseRef _List::makeDynamic(void)
-{
+BaseRef _List::makeDynamic(void) const {
     _List * Res = new _List;
-    checkPointer(Res);
-    //lData = nil;
-    memcpy ((char*)Res, (char*)this, sizeof (_List));
-    Res->nInstances = 1;
-    Res->lData = nil;
     Res->Duplicate (this);
     return Res;
 }
 
-void  _List::Replace (long index, BaseRef newObj, bool dup)
-{
+void  _List::Replace (long index, BaseRef newObj, bool dup) {
     if (index>=0 && index<lLength) {
-        DeleteObject (((BaseRef*)lData)[index]);
-        ((BaseRef*)lData)[index] = dup?newObj->makeDynamic():newObj;
+        BaseRef payload = dup?newObj->makeDynamic():newObj;
+        // important to do this BEFORE calling DeleteObject in case newObj == existing object
+        DeleteObject (((BaseRef*)list_data)[index]);
+        ((BaseRef*)list_data)[index] = payload;
     }
 }
 
 // Char* conversion
 //TODO: toFileStr should be ToFileStr to follow convention.
-void _List::toFileStr(FILE* dest, unsigned long)
-{
+void _List::toFileStr(FILE* dest, unsigned long) {
     fprintf (dest,"{");
   
     if (lLength) {
@@ -663,7 +579,7 @@ void _List::toFileStr(FILE* dest, unsigned long)
 // Char* conversion
 //TODO: toFileStr should be ToStr to follow convention.
 BaseRef _List::toStr(unsigned long) {
-    _String * s = new _String((unsigned long)20*(lLength+1),true);
+    _StringBuffer * s = new _StringBuffer((unsigned long)20*(lLength+1));
 
     (*s)<<'{';
   
@@ -676,7 +592,6 @@ BaseRef _List::toStr(unsigned long) {
     }
   
     (*s)<<'}';
-    s->Finalize();
     return s;
 }
 
