@@ -5,7 +5,7 @@ HyPhy - Hypothesis Testing Using Phylogenies.
 Copyright (C) 1997-now
 Core Developers:
   Sergei L Kosakovsky Pond (sergeilkp@icloud.com)
-  Art FY Poon    (apoon@cfenet.ubc.ca)
+  Art FY Poon    (apoon42@uwo.ca)
   Steven Weaver (sweaver@temple.edu)
   
 Module Developers:
@@ -38,16 +38,26 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
 #include "trie.h"
+#include "global_things.h"
+#include "hy_string_buffer.h"
 
 //----------------------------------------------------------------------------------------------------------------------
 
 _Trie::_Trie (const _String* alphabet) {
+    InitializeTrie(alphabet);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+void _Trie::InitializeTrie (const _String* alphabet) {
     SetAlphabet (alphabet, false);
     AppendNewInstance(new _SimpleList);
     payload << 0L;
     parents <<-1L;
+    inserted_values = 0UL;
 }
-       
+
+
 //----------------------------------------------------------------------------------------------------------------------
 
 _Trie::~_Trie (void){
@@ -61,20 +71,19 @@ void _Trie::Clear (bool all){
     payload.Clear(all);
     emptySlots.Clear(all);
     AppendNewInstance(new _SimpleList);
+    inserted_values = 0UL;
     payload << 0L;
     parents <<-1L;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
-_String _Trie::Alphabet (void)
-{
-    _String result (256L, true);
-    for (unsigned long charIndex = 0; charIndex < 256; charIndex++) {
-        if (charMap.lData[charIndex] >= 0)
+_String _Trie::Alphabet (void) const {
+    _StringBuffer result (256UL);
+    for (unsigned long charIndex = 0UL; charIndex < 256UL; charIndex++) {
+        if (charMap.list_data[charIndex] >= 0)
             result << char (charIndex);
     }
-    result.Finalize();
     return result;
 }
 
@@ -90,14 +99,14 @@ void _Trie::SetAlphabet (const _String* alphabet, bool doClear)
     if (alphabet) {
         charMap.Populate (256,-1,0);
         unsigned long charCounter = 0;
-        charMap.lData[0] = 1; // always allow the '\0' character
-        for (unsigned long charIndex = 0; charIndex < alphabet->sLength; charIndex++) {
-            charMap.lData [(unsigned char)alphabet->sData[charIndex]] = 1;
+        charMap.list_data[0] = 1; // always allow the '\0' character
+        for (unsigned long charIndex = 0; charIndex < alphabet->length(); charIndex++) {
+            charMap.list_data [(unsigned char)alphabet->char_at(charIndex)] = 1;
         }
         // now sort alphabetically
         for (unsigned long charIndex = 0; charIndex < 256; charIndex++) {
-            if (charMap.lData[charIndex] == 1)
-                charMap.lData[charIndex] = charCounter++;
+            if (charMap.list_data[charIndex] == 1)
+                charMap.list_data[charIndex] = charCounter++;
         }
     } else {
         charMap.Populate (256,0,1);
@@ -106,39 +115,40 @@ void _Trie::SetAlphabet (const _String* alphabet, bool doClear)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-BaseRef _Trie::makeDynamic (void) {
+BaseRef _Trie::makeDynamic (void) const {
     _Trie *newTrie = new _Trie ();
-    newTrie->Duplicate (newTrie);
+    newTrie->Duplicate (this);
     return newTrie;
           
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
-void _Trie::Duplicate (BaseRef storage) {
-    _Trie* newTrie = (_Trie*)storage;
-    _String myAlphabet = Alphabet();
-    newTrie->SetAlphabet (&myAlphabet, true);
-    newTrie->_List::Duplicate ((_List*)this);
-    newTrie->charMap.Duplicate (&charMap);
-    newTrie->emptySlots.Duplicate (&emptySlots);
-    newTrie->payload.Duplicate(&payload);
-    newTrie->parents.Duplicate(&parents);
+void _Trie::Duplicate (BaseRefConst t) {
+    _Trie* source = (_Trie*)t;
+    _String a (source->Alphabet());
+    SetAlphabet(&a, true);
+    _List::Duplicate ((_List*)source);
+    charMap.Duplicate (&source->charMap);
+    emptySlots.Duplicate (&source->emptySlots);
+    payload.Duplicate(&source->payload);
+    parents.Duplicate(&source->parents);
+    inserted_values = source->inserted_values;
           
 }        
     
 //----------------------------------------------------------------------------------------------------------------------
 
 long    _Trie::FindNextLetter (const char letter, const unsigned long current_index) const {
-    long letterKey = charMap.lData[(const unsigned char)letter];
+    long letterKey = charMap.list_data[(const unsigned char)letter];
     if (letterKey >= 0) {
-        _SimpleList* thisList = ((_SimpleList**)lData)[current_index];
+        _SimpleList* thisList = ((_SimpleList**)list_data)[current_index];
         letterKey = thisList->FindStepping (letterKey, 2, 0);
         if (letterKey < 0)
-            return HY_TRIE_NOTFOUND;
-        return thisList->lData[letterKey+1];
+            return kNotFound;
+        return thisList->list_data[letterKey+1];
     }
-    return HY_TRIE_INVALID_LETTER;
+    return kTrieInvalidLetter;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -147,7 +157,7 @@ long  _Trie::FindNextUnusedIndex (bool alloc){
     if (emptySlots.lLength) {
         long newIndex = emptySlots.Pop();
         if (alloc)
-            ((_SimpleList**)lData)[newIndex] = new _SimpleList;
+            ((_SimpleList**)list_data)[newIndex] = new _SimpleList;
         return newIndex;
     }  
     payload << 0;
@@ -163,34 +173,40 @@ long  _Trie::FindNextUnusedIndex (bool alloc){
 //----------------------------------------------------------------------------------------------------------------------
 
 long    _Trie::InsertNextLetter (const char letter, const unsigned long current_index) {
-    long letter_key = charMap.lData[(const unsigned char)letter];
+    long letter_key = charMap.list_data[(const unsigned char)letter];
     if (letter_key >= 0) {
         long next_index = FindNextUnusedIndex (letter != 0);
-        _SimpleList * currentList = ((_SimpleList**)lData)[current_index];
+        _SimpleList * currentList = ((_SimpleList**)list_data)[current_index];
         (*currentList) << letter_key;
         (*currentList) << next_index;
-        parents.lData[next_index] = current_index;
+        parents.list_data[next_index] = current_index;
         return next_index;
     }
-    return HY_TRIE_INVALID_LETTER;
+    return kTrieInvalidLetter;
 }
 
 
 //----------------------------------------------------------------------------------------------------------------------
-long     _Trie::FindKey (const _String& key, _SimpleList* path, bool prefixOK) const{
-    long current_index = 0,
-         next_index    = 0;
-    for (long k = 0; k <= key.sLength && current_index >= 0; k++){
-       next_index = FindNextLetter (key.sData[k], current_index);
+long     _Trie::FindKey (const _String& key, _SimpleList* path, bool prefixOK, unsigned long * start_index) const{
+    long current_index = 0L,
+         next_index    = 0L;
+    for (unsigned long k = start_index ? *start_index : 0UL; k <= key.length() && current_index >= 0L; k++){
+       next_index = FindNextLetter (key.char_at(k), current_index);
        if (path)
             (*path) << next_index;
        if (next_index < 0 && prefixOK) {
            next_index = FindNextLetter (0, current_index);
            current_index = next_index;
-           break;
+           if (start_index) {
+             *start_index = k;
+           }
+           return current_index;
        }
        current_index = next_index;
     }
+    if (start_index) {
+      *start_index = key.length();
+    }
     return current_index;
 }
 
@@ -209,23 +225,23 @@ long     _Trie::FindKey (const char key, bool prefixOK) const {
 //----------------------------------------------------------------------------------------------------------------------
 long     _Trie::GetValueFromString (const _String& key){
     long keyIndex = FindKey (key);
-    if (keyIndex != HY_TRIE_NOTFOUND) {
+    if (keyIndex != kNotFound) {
         return GetValue (keyIndex);
     }
-    return HY_TRIE_NOTFOUND;
+    return kNotFound;
 }
 
 
 //----------------------------------------------------------------------------------------------------------------------
 void     _Trie::UpdateValue(const long key, const long value) {
     if (key >= 0 && key < payload.lLength)
-        payload.lData[key] = value;
+        payload.list_data[key] = value;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
-long     _Trie::GetValue(const long key) {
+long     _Trie::GetValue(const long key) const{
     if (key >= 0 && key < payload.lLength)
-        return payload.lData[key];
+        return payload.list_data[key];
     
     return 0L;
 }
@@ -236,7 +252,7 @@ long    _Trie::Insert (const char* key, const long value, bool return_index) {
     _String key_string(key);
     long ret_value = Insert  (key_string, value);
     if (ret_value >= 0 && return_index == false) {
-        return key_string.sLength;
+        return key_string.length();
     }
     return ret_value;
 }
@@ -251,39 +267,51 @@ _Trie& _Trie::operator < (const char* key) {
 //----------------------------------------------------------------------------------------------------------------------
 
 long    _Trie::Insert (const _String& key, const long value) {
-    // the root is always at index 0
-    long current_index  = 0, 
-         current_char   = 0,
-         next_index     = FindNextLetter(key.sData[current_char++], current_index);
+    return InsertExtended (key, value);
+}
     
-    while (next_index >= 0 && current_char <= key.sLength) {
+//----------------------------------------------------------------------------------------------------------------------
+
+long    _Trie::InsertExtended (const _String& key, const long value, bool update_value, bool * did_insert) {
+    // the root is always at index 0
+    long current_index  = 0L,
+         current_char   = 0L,
+         next_index     = FindNextLetter(key.char_at (current_char++), current_index);
+    
+    while (next_index >= 0L && current_char <= key.length()) {
         current_index = next_index;
-        next_index     = FindNextLetter(key.sData[current_char++], current_index);
+        next_index     = FindNextLetter(key.char_at (current_char++), current_index);
     }
     
-    if (next_index == HY_TRIE_INVALID_LETTER)
-        return HY_TRIE_INVALID_LETTER;
+    if (next_index == kTrieInvalidLetter)
+        return kTrieInvalidLetter;
     
-    if (current_char == key.sLength && next_index >= 0)
+    if (current_char > key.length() ) { // key already present
+        if (update_value) {
+            UpdateValue (next_index, value);
+        }
+        if (did_insert) *did_insert = false;
         return next_index;
+    }
     
     current_char --;
     
     // validate the rest of the string
     
-    for (long k = current_char; k <= key.sLength; k++) {
-        if (charMap[key.sData[k]] < 0)
-            return HY_TRIE_INVALID_LETTER;
+    for (long k = current_char; k <= key.length(); k++) {
+        if (charMap[key.char_at(k)] < 0)
+            return kTrieInvalidLetter;
     }
     
      // insert the rest of the string
-    for (; current_char <= key.sLength; current_char++) {
+    for (; current_char <= key.length(); current_char++) {
         //printf ("\nInserting %c\n", key.sData[current_char]);
-        current_index = InsertNextLetter (key.sData[current_char], current_index);      
+        current_index = InsertNextLetter (key.char_at (current_char), current_index);
         //DumpRaw ();
-   }
-    
+    }
+    inserted_values ++;
     UpdateValue (current_index, value);
+    if (did_insert) *did_insert = true;
 
     return current_index;
 }
@@ -293,10 +321,11 @@ long    _Trie::Insert (const _String& key, const long value) {
 unsigned long    _Trie::Insert (const _List& key, const _SimpleList* values) {
     unsigned long how_many = 0;
     for (long k = 0; k < key.lLength; k++) {
-        _String serializedKey ((_String*)((BaseRef*)key.lData)[k]->toStr());
+        _String serializedKey ((_String*)((BaseRef*)key.list_data)[k]->toStr());
+        bool did_insert = false;
         
-        long this_index = Insert (serializedKey, values?values->lData[k]:0);
-        if (this_index >= 0) {
+        InsertExtended (serializedKey, values?values->list_data[k]:0, false, &did_insert);
+        if (did_insert) {
             how_many ++;
         }
     }
@@ -310,17 +339,18 @@ bool    _Trie::Delete (const _String& key){
     if (found_key >= 0) {
         // now traverse the history list backwards and delete all keys that have no children
         for (long k = history.lLength-1; k>=0; k--) {
-            _SimpleList * current_list = ((_SimpleList**)lData)[history.lData[k]];
+            _SimpleList * current_list = ((_SimpleList**)list_data)[history.list_data[k]];
             if (current_list == nil || current_list->lLength <= 1){
-                emptySlots << history.lData[k];
-                payload.lData[history.lData[k]] = 0L;
-                parents.lData[history.lData[k]] = -1L;
-                _SimpleList * parentList = ((_SimpleList**)lData)[history.lData[k-1]];
-                unsigned long parentNode = parentList->FindStepping (history.lData[k],2, 1) - 1;
+                emptySlots << history.list_data[k];
+                payload.list_data[history.list_data[k]] = 0L;
+                parents.list_data[history.list_data[k]] = -1L;
+                _SimpleList * parentList = ((_SimpleList**)list_data)[history.list_data[k-1]];
+                unsigned long parentNode = parentList->FindStepping (history.list_data[k],2, 1) - 1;
                 parentList->Delete (parentNode);
                 parentList->Delete (parentNode);
                 DeleteObject (current_list);
-                ((_SimpleList**)lData)[history.lData[k]] = nil;
+                ((_SimpleList**)list_data)[history.list_data[k]] = nil;
+                inserted_values--;
             }
         }
         return true;
@@ -338,7 +368,7 @@ bool    _Trie::Delete (const char* key){
 unsigned long     _Trie::Delete (const _List& key){
      unsigned long how_many = 0;
     for (long k = 0; k < key.lLength; k++) {
-        _String serializedKey ((_String*)((BaseRef*)key.lData)[k]->toStr());
+        _String serializedKey ((_String*)((BaseRef*)key.list_data)[k]->toStr());
         
         long this_index = Delete (serializedKey);
         
@@ -350,22 +380,28 @@ unsigned long     _Trie::Delete (const _List& key){
 }
 
  //----------------------------------------------------------------------------------------------------------------------
-_String*         _Trie::RetrieveStringFromPath (const _SimpleList& path, _String* alphabet) {
-    _String* this_string = new _String (128L,true),
-           * my_alph      = alphabet? alphabet : new _String (Alphabet());
-    
+_String*         _Trie::RetrieveStringFromPath (const _SimpleList& path, _String const * alphabet) const {
+  
+    _StringBuffer * this_string = new _StringBuffer (128UL);
+  
+  
+    _String     * local_alph      = nil;
+    _String  const * read_from = alphabet;
+  
+    if (!read_from) {
+      local_alph = new _String (Alphabet());
+      read_from = local_alph;
+    }
+  
     
     for (long k = 0; k < path.lLength - 4; k+=2) {
-         _SimpleList* current_list     = ((_SimpleList**)lData)[path.lData[k]];
-         long         current_position = path.lData[k+1];
-         (*this_string) << my_alph->sData[current_list->lData[current_position]];
+         _SimpleList* current_list     = ((_SimpleList**)list_data)[path.list_data[k]];
+         long         current_position = path.list_data[k+1];
+         (*this_string) << read_from->char_at (current_list->list_data[current_position]);
     }
     
-    this_string->Finalize();
-    
-    if (!alphabet)
-        DeleteObject(my_alph);
-    
+  
+    DeleteObject(local_alph);
     return this_string;
 }
   
@@ -375,9 +411,9 @@ void    _Trie::DumpRaw() {
     for (long k = 0; k < lLength; k++) {
         if (emptySlots.Find(k) < 0) {
             printf ("Position %ld:\n", k);
-            _SimpleList * this_list = ((_SimpleList**)lData)[k];
+            _SimpleList * this_list = ((_SimpleList**)list_data)[k];
             for (long m = 0; m < this_list->lLength; m+=2) {
-                printf ("'%c'(%ld) -> %ld\n", (char)this_list->lData[m], this_list->lData[m], this_list->lData[m+1]);
+                printf ("'%c'(%ld) -> %ld\n", (char)this_list->list_data[m], this_list->list_data[m], this_list->list_data[m+1]);
             }
             
         } else {
@@ -389,37 +425,35 @@ void    _Trie::DumpRaw() {
 //----------------------------------------------------------------------------------------------------------------------
 
 BaseRef     _Trie::toStr(unsigned long) {
-    _String         * serialized = new _String (128L, true),
-                      alph       = Alphabet();
-                      
+
+    _StringBuffer * serialized = new _StringBuffer (128UL);
+    _String const   alph       = Alphabet();
+  
     _SimpleList       traversal_history, 
                         // 2 indices per entry: node and current position (in multiples of 2)
-                      *root_list = ((_SimpleList**)lData)[0];
+                      *root_list = ((_SimpleList**)list_data)[0];
                       
     traversal_history << 0; traversal_history << 0;
     
     bool doComma = false;
     
     (*serialized) << '{';
-    while (!(traversal_history.lLength == 2 && traversal_history.lData[1] == root_list->lLength)) {
-        _SimpleList* current_list = ((_SimpleList**)lData)[traversal_history.lData[traversal_history.lLength-2]];
-        long current_position = traversal_history.lData[traversal_history.lLength-1];
+    while (!(traversal_history.lLength == 2 && traversal_history.list_data[1] == root_list->lLength)) {
+        _SimpleList* current_list = ((_SimpleList**)list_data)[traversal_history.list_data[traversal_history.lLength-2]];
+        long current_position = traversal_history.list_data[traversal_history.lLength-1];
         // if current list is empty, then generate a string based on the path, and advance up the chain
         if (current_list && current_list->lLength) {
             if (current_position < current_list->lLength) {
-                traversal_history << current_list->lData[current_position+1];
+                traversal_history << current_list->list_data[current_position+1];
                 traversal_history << 0;
             } else {
                 traversal_history.Pop();
                 traversal_history.Pop();
-                traversal_history.lData[traversal_history.lLength-1] += 2; // advance the counter in the parent
+                traversal_history.list_data[traversal_history.lLength-1] += 2; // advance the counter in the parent
            }
         } else {
             _String * this_string = RetrieveStringFromPath(traversal_history, &alph);
-            (*serialized) << '"';
-            (*serialized) << this_string;
-            (*serialized) << "\":";
-            (*serialized) << _String (GetValue (traversal_history.lData[traversal_history.lLength-2]));
+            (*serialized) << '"' << this_string << "\":" << _String (GetValue (traversal_history.list_data[traversal_history.lLength-2]));
             if (doComma) {
                 (*serialized) << ',';
             } else {
@@ -428,18 +462,17 @@ BaseRef     _Trie::toStr(unsigned long) {
             (*serialized) << '\n';
             traversal_history.Pop();
             traversal_history.Pop();            
-            traversal_history.lData[traversal_history.lLength-1] += 2; // advance the counter in the parent
+            traversal_history.list_data[traversal_history.lLength-1] += 2; // advance the counter in the parent
        }
     }
     
     (*serialized) << '}';
-    serialized->Finalize();
     return serialized;
 }
  
 //----------------------------------------------------------------------------------------------------------------------
 
-_String  _Trie::RetrieveKeyByPayload (const long key){
+_String  const _Trie::RetrieveKeyByPayload (const long key){
     long key_index = payload.Find (key);
     if (key_index >= 0) {
         _SimpleList parent_indices,
@@ -447,7 +480,7 @@ _String  _Trie::RetrieveKeyByPayload (const long key){
         long keyer = key_index;
         do{
             parent_indices << keyer;
-            keyer = parents.lData[keyer];
+            keyer = parents.list_data[keyer];
         }
         while (keyer > 0);
         parent_indices << 0;
@@ -455,8 +488,8 @@ _String  _Trie::RetrieveKeyByPayload (const long key){
         
         
         for (long i = 0; i < parent_indices.lLength-1; i++) {
-            traversal_history << parent_indices.lData[i];
-            traversal_history << (((_SimpleList**)lData)[parent_indices.lData[i]])->FindStepping (parent_indices.lData[i+1],2,1)-1;
+            traversal_history << parent_indices.list_data[i];
+            traversal_history << (((_SimpleList**)list_data)[parent_indices.list_data[i]])->FindStepping (parent_indices.list_data[i+1],2,1)-1;
         }
         traversal_history << key_index;
         traversal_history << 0L;
@@ -464,7 +497,7 @@ _String  _Trie::RetrieveKeyByPayload (const long key){
         return _String(RetrieveStringFromPath(traversal_history, &alph));
         
     }
-    return emptyString;
+    return hy_global::kEmptyString;
 }
 
 
