@@ -4,9 +4,9 @@
  
  Copyright (C) 1997-now
  Core Developers:
- Sergei L Kosakovsky Pond (spond@ucsd.edu)
- Art FY Poon    (apoon@cfenet.ubc.ca)
- Steven Weaver (sweaver@ucsd.edu)
+ Sergei L Kosakovsky Pond (spond@temple.edu)
+ Art FY Poon    (apoon42@uwo.ca)
+ Steven Weaver (sweaver@temple.edu)
  
  Module Developers:
  Lance Hepler (nlhepler@gmail.com)
@@ -38,47 +38,47 @@
  */
 
 #include <stdio.h>
-#include "likefunc.h"
 
 
-
-#ifndef __HYPHY_NO_CURL__
-//#define __HYPHYCURL__
-#endif
-
 #ifdef  __HYPHYCURL__
-#include <curl/curl.h>
+    #include <curl/curl.h>
 #endif
 
+#include "global_object_lists.h"
+#include "likefunc.h"
+#include "hy_string_buffer.h"
+#include "hbl_env.h"
+
+using    namespace hy_global;
+
+
 #ifdef          __HYPHYMPI__
 _String         preserveSlaveNodeState ("PRESERVE_SLAVE_NODE_STATE"),
                 MPI_NEXUS_FILE_RETURN  ("MPI_NEXUS_FILE_RETURN");
 
-int             _hy_mpi_node_rank;
 
 void            mpiNormalLoop    (int, int, _String &);
 void            mpiOptimizerLoop (int, int);
 
 void            mpiBgmLoop (int, int);
-_SimpleList     mpiNodesThatCantSwitch;
 #endif
 
+extern _List batchLanguageFunctionNames;
+
 
 //_________________________________________________________________________
 
-size_t url2File   (void *ptr, size_t size, size_t nmemb, void *stream)
-{
+size_t url2File   (void *ptr, size_t size, size_t nmemb, void *stream) {
     return fwrite (ptr, size, nmemb, (FILE*)stream);
 }
 
 //_________________________________________________________________________
 
-size_t url2String (void *ptr, size_t size, size_t nmemb, void *stream)
-{
-    _String * s = (_String*)stream;
+size_t url2String (void *ptr, size_t size, size_t nmemb, void *stream) {
+    _StringBuffer * s = (_StringBuffer*)stream;
     char    * p = (char*)ptr;
 
-    for (unsigned long k=0; k<size*nmemb; k++) {
+    for (unsigned long k=0UL; k<size*nmemb; k++) {
         (*s) << p[k];
     }
 
@@ -87,28 +87,30 @@ size_t url2String (void *ptr, size_t size, size_t nmemb, void *stream)
 
 //_________________________________________________________________________
 
-bool    Get_a_URL (_String& urls, _String* fileName)
-{
+bool    Get_a_URL (_String& urls, _String*
+#ifdef __HYPHYCURL__
+                   fileName
+#endif
+                   ) {
 #ifdef __HYPHYCURL__
     CURL *curl;
     CURLcode res ;
     curl = curl_easy_init ();
     FILE   * f = nil;
-    _String* s = nil;
+    _StringBuffer * s = nil;
     char cErr [CURL_ERROR_SIZE+1];
     if(curl) {
         if (fileName) {
-            f = fopen (fileName->sData,"wb");
+            f = fopen (fileName->get_str(),"wb");
             if (!f) {
                 urls = _String ("Failed to open ") & *fileName & " for writing";
                 return false;
             }
         } else {
-            s = new _String (8192, true);
-            checkPointer (s);
+            s = new _StringBuffer (8192UL);
         }
 
-        curl_easy_setopt (curl, CURLOPT_URL, urls.sData );
+        curl_easy_setopt (curl, CURLOPT_URL, urls.get_str() );
         curl_easy_setopt (curl, CURLOPT_ERRORBUFFER, cErr);
 
         //Do not check peer certificate, since we only ever get urls
@@ -122,12 +124,11 @@ bool    Get_a_URL (_String& urls, _String* fileName)
         }
 
         _String ver (GetVersionString());
-        curl_easy_setopt (curl, CURLOPT_USERAGENT, ver.sData);
+        curl_easy_setopt (curl, CURLOPT_USERAGENT, ver.get_str());
         //curl_easy_setopt (curl, CURLOPT_VERBOSE, 1);
         curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, (void*)(f?url2File:url2String));
-        _Parameter vbl = 0.0;
-        checkParameter (VerbosityLevelString,vbl,0.0);
-        if (vbl<0.5) {
+      
+        if (hy_env::EnvVariableTrue (VerbosityLevelString)) {
             curl_easy_setopt (curl,CURLOPT_NOPROGRESS,1);
         }
         res = curl_easy_perform (curl);
@@ -136,7 +137,6 @@ bool    Get_a_URL (_String& urls, _String* fileName)
         if (f) {
             fclose (f);
         } else {
-            s->Finalize();
             urls = *s;
             DeleteObject (s);
         }
@@ -159,40 +159,37 @@ bool    Get_a_URL (_String& urls, _String* fileName)
 
 //____________________________________________________________________________________
 
-_String*    StringFromConsole   (bool)
-{
-    _String * returnme = new _String (32L, true);
+_String *    StringFromConsole   () {
+    fflush(stdout);
+    _StringBuffer * returnme = new _StringBuffer (32UL);
 #if not defined __HEADLESS__ && not defined _MINGW32_MEGA_
     int       readAChar;
     while    ((readAChar = getc(stdin)) != '\n') {
         if (readAChar == EOF) {
-            CheckReceptacleAndStore (&hasEndBeenReached,empty,false,new _Constant (1.), false);
+            hy_env::EnvVariableSet(hy_env::end_of_file, new HY_CONSTANT_TRUE, false);
             break;
         }
-        *returnme << readAChar;
+        *returnme << (char)readAChar;
     }
 #else
     WarnError ("Unhandled standard input interaction in StringFromConsole for headless HyPhy");
     return NULL;
 #endif
-    returnme->Finalize ();
     return returnme;
 }
 
 //__________________________________________________________________________________
 
-void    StringToConsole (_String & s,  _SimpleList *)
-{
-    BufferToConsole ((const char*)s.sData);
+void    StringToConsole (_String const & s,  void * extra) {
+    BufferToConsole ((const char*)s, extra);
 }
 
 
 //__________________________________________________________________________________
 
-void    BufferToConsole (const char* s, _SimpleList *)
-{
+void    BufferToConsole (const char* s, void * extra) {
 #ifdef __HYPHYMPI__
-    if (_hy_mpi_node_rank == 0)
+    if (hy_mpi_node_rank == 0)
 #endif
 #ifdef __HEADLESS__
         if (globalInterfaceInstance) {
@@ -200,15 +197,25 @@ void    BufferToConsole (const char* s, _SimpleList *)
             globalInterfaceInstance->PushOutString(&st);
         }
 #else
-        printf ("%s",s);
+  if (extra) {
+    fprintf ((FILE*)extra, "%s",s);
+  } else {
+    printf ("%s",s);
+    fflush(stdout);
+  }
 #endif
 }
 
 //__________________________________________________________________________________
 
-void    NLToConsole (void)
-{
-    BufferToConsole ("\n");
+void    NLToConsole (void * extra) {
+    BufferToConsole ("\n", extra);
+}
+
+//__________________________________________________________________________________
+
+void    ObjectToConsole (BaseRef obj,  void * extra) {
+    StringToConsole (_String (((_String*)obj->toStr())), extra );
 }
 
 #endif
@@ -222,75 +229,81 @@ void mpiNormalLoop    (int rank, int size, _String & baseDir)
 
     ReportWarning ("[MPI] Entered mpiNormalLoop");
 
-    _String* theMessage     = MPIRecvString (-1,senderID),  // listen for messages from any node
-             * resStr        = nil;
+    _String* theMessage     = MPIRecvString (-1,senderID);   // listen for messages from any node
+    _StringBuffer * resStr        = nil;
 
-    while (theMessage->sLength) {
-        setParameter    (mpiNodeID,    (_Parameter)rank);
-        setParameter    (mpiNodeCount, (_Parameter)size);
-
-        //ReportWarning (*theMessage);
+    while (theMessage->nonempty()) {
+        hy_env :: EnvVariableSet (hy_env ::mpi_node_id, new _Constant (rank), false);
+        hy_env :: EnvVariableSet (hy_env ::mpi_node_count, new _Constant (size), false);
+ 
         DeleteObject (resStr);
         resStr       = nil;
-        if (theMessage->startswith (mpiLoopSwitchToOptimize) ) {
-            hyphyMPIOptimizerMode   = theMessage->Cut(mpiLoopSwitchToOptimize.sLength,-1).toNum();
+        if (theMessage->BeginsWith (mpiLoopSwitchToOptimize) ) {
+            hyphyMPIOptimizerMode   = theMessage->Cut(mpiLoopSwitchToOptimize.length(),kStringEnd).to_long();
 
             ReportWarning           (_String("[MPI] Switched to mpiOptimizer loop with mode ") & hyphyMPIOptimizerMode);
             MPISendString           (mpiLoopSwitchToOptimize,senderID);
             mpiOptimizerLoop        (rank,size);
             ReportWarning           ("[MPI] Returned from mpiOptimizer loop");
             hyphyMPIOptimizerMode   = _hyphyLFMPIModeNone;
-            pathNames               && & baseDir;
-        } else if ( theMessage->Equal (&mpiLoopSwitchToBGM) ) {
+            PushFilePath(baseDir, false, false);
+        } else if ( *theMessage == mpiLoopSwitchToBGM) {
             ReportWarning       ("[MPI] Received signal to switch to mpiBgmLoop");
             MPISendString       (mpiLoopSwitchToBGM, senderID); // feedback to source to confirm receipt of message
             mpiBgmLoop          (rank, size);
             ReportWarning       ("[MPI] Returned from mpiBgmLoop");
         } else {
-            if (theMessage->beginswith ("#NEXUS")) {
+            if (theMessage->BeginsWith ("#NEXUS")) {
                 _String             msgCopy (*theMessage);
-                ReportWarning       ("[MPI] Received a function to optimize");
+                ReportWarning       ("[MPI] Received a likelihood function");
+                //ReportWarning       (msgCopy);
                 ReadDataSetFile     (nil,true,theMessage);
-                ReportWarning       ("[MPI] Done with the optimization");
+                ReportWarning       ("[MPI] Read/optimized the likelihood function");
                 _Variable*          lfName = FetchVar(LocateVarByName(MPI_NEXUS_FILE_RETURN));
 
                 if (lfName) {
-                    resStr = (_String*)(lfName->Compute()->toStr());
-                } else {
+                    resStr = new _StringBuffer ((_String*)lfName->Compute()->toStr());
+                 } else {
                     _FString        *lfID = (_FString*)FetchObjectFromVariableByType (&lf2SendBack, STRING);
 
                     if (!lfID) {
-                        FlagError (_String("[MPI] Malformed MPI likelihood function optimization request - did not specify the LF name to return in variable ") & lf2SendBack & ".\n\n\n" );
+                        HandleApplicationError (_String("[MPI] Malformed MPI likelihood function optimization request - did not specify the LF name to return in variable ") & lf2SendBack & ".\n\n\n" );
                         break;
                     }
 
-                    long f = likeFuncNamesList.Find (lfID->theString);
+                    long type = HY_BL_LIKELIHOOD_FUNCTION, index;
+                    
+                    _LikelihoodFunction *lf = (_LikelihoodFunction *)hyphy_global_objects::_HYRetrieveBLObjectByName    (lfID->get_str(), type, &index, false, false);
 
-                    if (f<0) {
-                        FlagError ("[MPI] Malformed MPI likelihood function optimization request - LF name to return did not refer to a well-defined likelihood function.\n\n\n");
+                    if (lf == nil) {
+                        HandleApplicationError (_String("[MPI] Malformed MPI likelihood function optimization request - '") & lfID->get_str() &"' did not refer to a well-defined likelihood function.\n\n\n");
                         break;
                     }
-                    _Parameter      pv;
-                    checkParameter (shortMPIReturn, pv ,0);
-                    resStr       = (_String*)checkPointer(new _String (1024L,true));
-                    ((_LikelihoodFunction*)likeFuncList (f))->SerializeLF(*resStr,pv>0.5?_hyphyLFSerializeModeShortMPI:_hyphyLFSerializeModeLongMPI);
-                    resStr->Finalize();
+                  
+                    resStr       = new _StringBuffer (1024UL);
+                    lf->SerializeLF(*resStr,hy_env::EnvVariableTrue (hy_env::short_mpi_return) ? _hyphyLFSerializeModeShortMPI:_hyphyLFSerializeModeLongMPI);
                 }
             } else {
+                //ReportWarning(_String ("[MPI] Received commands\n") & *theMessage & "\n");
                 _ExecutionList exL (*theMessage);
-                _PMathObj res = exL.Execute();
-                resStr = res?(_String*)res->toStr():new _String ("0");
+                //ReportWarning (_String ((_String*)batchLanguageFunctionNames.toStr()));
+                HBLObjectRef res = exL.Execute();
+                if (res) {
+                  resStr = new _StringBuffer ((_String*)res->toStr());
+                } else {
+                  resStr = new _StringBuffer ("0");
+                }
             }
 
-            checkPointer (resStr);
             MPISendString(*resStr,senderID);
-
-            _Parameter      keepState = 0.0;
-            checkParameter  (preserveSlaveNodeState, keepState, 0.0);
-
-            if (keepState < 0.5) {
+            
+            if (hy_env::EnvVariableTrue (preserveSlaveNodeState) == false) {
                 PurgeAll (true);
-                pathNames && & baseDir;
+                InitializeGlobals ();
+                PushFilePath(baseDir, false, false);
+                ReportWarning("Reset node state");
+            } else {
+                ReportWarning("Preserved node state");
             }
         }
         DeleteObject (theMessage);
@@ -314,33 +327,36 @@ void mpiOptimizerLoop (int rank, int size)
 
     //printf ("Node %d waiting for a string\n", rank);
     _String* theMessage = MPIRecvString (-1,senderID);
-    while (theMessage->sLength) {
-        if (theMessage->beginswith ("#NEXUS")) {
+    while (theMessage->nonempty()) {
+        if (theMessage->BeginsWith ("#NEXUS")) {
             //ReportWarning (*theMessage);
             ReadDataSetFile (nil,true,theMessage);
             if (likeFuncNamesList.lLength!=1) {
-                FlagError ("[MPI] Malformed MPI likelihood function paraller optimizer startup command. Exactly ONE valid LF must be defined.n\n\n");
+                HandleApplicationError ("[MPI] Malformed MPI likelihood function paraller optimizer startup command. Exactly ONE valid LF must be defined.n\n\n");
                 break;
             }
 
             // send back the list of independent variables
 
             _LikelihoodFunction * theLF = (_LikelihoodFunction*)likeFuncList (0);
-            if (hyphyMPIOptimizerMode == _hyphyLFMPIModeREL && theLF->CountObjects (4)) {
-                FlagError (_String("[MPI] Likelihood functions spawned off to slave MPI nodes can't have category variables.n\n\n"));
+            if (hyphyMPIOptimizerMode == _hyphyLFMPIModeREL && theLF->CountObjects (kLFCountCategoryVariables)) {
+                HandleApplicationError (_String("[MPI] Likelihood functions spawned off to slave MPI nodes can't have category variables.n\n\n"));
                 break;
             }
 
-            _SimpleList* ivl = & theLF->GetIndependentVars();
-            _String      variableSpec (128L, true);
+            _SimpleList const * ivl = & theLF->GetIndependentVars();
+  
+          
+            _StringBuffer      variableSpec (128UL);
 
-            (variableSpec) << LocateVar(ivl->lData[0])->GetName();
 
+            (variableSpec) << LocateVar(ivl->list_data[0])->GetName();
+            
             for (long kk = 1; kk < ivl->lLength; kk++) {
-                (variableSpec) << ';';
-                (variableSpec) << LocateVar(ivl->lData[kk])->GetName();
+              (variableSpec) << ';';
+              (variableSpec) << LocateVar(ivl->list_data[kk])->GetName();
             }
-            variableSpec.Finalize();
+          
             ReportWarning         (_String("[MPI] Sending back the following variable list\n") & variableSpec);
             MPISendString         (variableSpec,senderID);
             theLF->PrepareToCompute();
@@ -348,6 +364,8 @@ void mpiOptimizerLoop (int rank, int size)
                                                hyphyMPIOptimizerMode  == _hyphyLFMPIModeSiteTemplate));
             theLF->DoneComputing();
             PurgeAll (true);
+            InitializeGlobals ();
+            ReportWarning("Reset node state at the end of MPI optimizaer loop");
         }
         DeleteObject (theMessage);
         theMessage = MPIRecvString (-1,senderID);
@@ -367,16 +385,15 @@ void mpiBgmLoop (int rank, int size)
     // receive serialized Bgm
     _String* theMessage = MPIRecvString (-1, senderID);
 
-    while (theMessage->sLength) {
+    while (theMessage->nonempty()) {
         _ExecutionList  exL (*theMessage);
-        _PMathObj       res = exL.Execute();    // should send this process into CacheNodeScores()
+        HBLObjectRef       res = exL.Execute();    // should send this process into CacheNodeScores()
 
         resStr = res ? (_String*)res->toStr() : new _String ("0");
         ReportWarning (_String ("MPI Node: ") & (long)rank & " executed HBL with result:\n" & resStr);
 
         if (bgmNamesList.lLength < 1) {
-            _String errMsg ("Malformed HBL. No valid BGM has been defined.\n");
-            FlagError (errMsg);
+            HandleApplicationError ("Malformed HBL. No valid BGM has been defined.\n");
             break;
         }
     }
@@ -384,4 +401,3 @@ void mpiBgmLoop (int rank, int size)
     DeleteObject (theMessage);
 }
 #endif
-
